#!/usr/bin/python
# Copyright (c) 2013, Red Hat, Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Stanislav Ochotnicky <sochotnicky@redhat.com>

import codecs
import os
import errno
import re
from StringIO import StringIO
import xml.dom.minidom

import lxml
import lxml.etree as ET
from lxml.etree import ElementTree, Element, SubElement

from javapackages.artifact import ArtifactValidationException

class XMvnConfigException(Exception):
    pass

class XMvnConfig(object):
    """
    Class for modifying XMvn configuration
    """

    INDEX_PATH = os.path.join(".xmvn", "javapackages-rule-index")
    CONFIG_DIR = os.path.join(".xmvn", "config.d")
    XMLNS = "http://fedorahosted.org/xmvn/CONFIG/0.6.0"

    def __init__(self):
        self.templateXML ="""<?xml version='1.0' encoding='utf-8'?>
<configuration xmlns="http://fedorahosted.org/xmvn/CONFIG/0.6.0">
{content}
</configuration>
"""
        try:
            os.makedirs(XMvnConfig.CONFIG_DIR, 0755)
        except OSError:
            # If directory already exists then it's OK.  If it doesn't
            # then creating the index file below will fail anyways.
            pass

        try:
            with open(XMvnConfig.INDEX_PATH) as index:
                self.index = int(index.read()) + 1
        except IOError as e:
            if e.errno == errno.ENOENT:
                self.index = 1
            else:
                raise e

    def __write_index(self):
        with open(XMvnConfig.INDEX_PATH, 'w') as index:
            index.write(str(self.index))
            self.index = self.index + 1

    def __get_current_config(self):
        fname = 'javapackages-config-{index:05d}.xml'.format(index=self.index)
        return os.path.join(XMvnConfig.CONFIG_DIR,
                            fname)

    def __init_xml(self, content=""):
        self.__write_index()

        xmlbuf = StringIO()
        xmlbuf.write(self.templateXML.format(content=content))

        root = ET.fromstring(xmlbuf.getvalue())
        root.append(ET.Comment("XMvn configuration file generated by "
                               "javapackages.xmvn_config  (part of "
                               "javapackages-tools)"))
        return root

    def __prettify_element(self, elem):
        xmlbuf = StringIO()
        et = ElementTree()
        et._setroot(elem)
        et.write(xmlbuf,
                 xml_declaration=True,
                 encoding = 'utf-8',
                 method = "xml",
                 pretty_print=True)
        return  xmlbuf.getvalue()


    def __write_xml(self, path, root):
        xmlstr = self.__prettify_element(root)
        with codecs.open(path, 'w+', "utf-8") as fout:
            fout.write(xmlstr)


    def __add_config(self, level1, level2, level3=None, content=None):
        if not content:
            raise Exception("Provide content as keyword argument")

        confpath = self.__get_current_config()
        root = self.__init_xml()

        level1 = SubElement(root, level1)
        level2 = SubElement(level1, level2)
        cont_level = level2
        if level3:
            cont_level = SubElement(level2, level3)

        if isinstance(content, basestring):
            cont_level.text = content
        elif isinstance(content, list):
            for elem in content:
                cont_level.append(elem)
        else:
            cont_level.append(content)

        self.__write_xml(confpath, root)

    def __count_backreferences(self, s):
        """
        Return maximum number of backreference used in string s
        """
        backref_re = re.compile('@(\d+)')
        backref_nos = [int(x) for x in backref_re.findall(s)]
        if backref_nos:
            return max((int(x) for x in backref_re.findall(s)))
        return 0

    def __count_wildcard_groups(self, s):
        """
        Return number of wildcard groups used in string s
        """
        left = s.count('{')
        right = s.count('}')

        if left != right:
            raise Exception("Number of opening and closing "
                            "parenthesis for groups of wildcard "
                            "matching is different.")
        return left
    def add_aliases(self, artifact, aliases):
        """
        Adds alias artifacts for given main artifact

        artifact -- main Artifact for which aliases are being provided
        aliases -- list of alternate Artifact representations
        """
        artifact.validate(allow_backref=False)
        wild_groups = self.__count_wildcard_groups(artifact.get_rpm_str())
        main = artifact.get_xml_element(root="artifactGlob")
        elems = [main]
        aelem = Element("aliases")
        for alias in aliases:
            alias.validate(allow_empty=False, allow_wildcards=False)
            backrefs = self.__count_backreferences(alias.get_rpm_str())
            if backrefs > wild_groups:
                raise ArtifactValidationException("Number of backrefenreces "
                                                  "is higher than wildcard "
                                                  "groups.")
            aelem.append(alias.get_xml_element(root="alias"))
        elems.append(aelem)
        self.__add_config("artifactManagement", "rule", content=elems)

    def add_compat_versions(self, artifact, versions):
        """
        Change where on filesystem given artifact is installed

        artifact -- Artifact to be modified
        versions -- list of compat versions for given artifact
        """
        artifact.validate(allow_backref=False)
        wild_groups = self.__count_wildcard_groups(artifact.get_rpm_str(artifact.version))
        main = artifact.get_xml_element(root="artifactGlob")
        elems = [main]
        velem = Element("versions")
        for version in versions:
            backrefs = self.__count_backreferences(version)
            if backrefs > wild_groups:
                raise ArtifactValidationException("Number of backrefenreces "
                                                  "is higher than wildcard "
                                                  "groups.")
            ve = SubElement(velem, "version")
            ve.text = version
        elems.append(velem)
        self.__add_config("artifactManagement", "rule", content=elems)

    def add_file_mapping(self, artifact, paths):
        """
        Change where on filesystem given artifact is installed

        artifact -- Artifact to be modified
        paths -- list of paths for given artifact
        """
        artifact.validate(allow_backref=False)
        wild_groups = self.__count_wildcard_groups(artifact.get_rpm_str())
        main = artifact.get_xml_element(root="artifactGlob")
        elems = [main]
        felem = Element("files")
        if not [path for path in paths if not os.path.isabs(path)]:
            raise XMvnConfigException("At least one path must be relative")
        for path in paths:
            backrefs = self.__count_backreferences(path)
            if backrefs > wild_groups:
                raise ArtifactValidationException("Number of backrefenreces "
                                                  "is higher than wildcard "
                                                  "groups.")
            pe = SubElement(felem, "file")
            pe.text = path
        elems.append(felem)
        self.__add_config("artifactManagement", "rule", content=elems)

    def add_package_mapping(self, artifact, package, optional=False):
        """
        Change which package given artifact belongs to

        artifact -- Artifact to be modified
        package -- subpackage name where artifact belongs
        """
        artifact.validate(allow_backref=False)
        wild_groups = self.__count_wildcard_groups(artifact.get_rpm_str())
        main = artifact.get_xml_element(root="artifactGlob")
        backrefs = self.__count_backreferences(package)
        if backrefs > wild_groups:
            raise ArtifactValidationException("Number of backrefenreces "
                                              "is higher than wildcard "
                                              "groups.")
        elems = [main]
        if optional:
            opt = Element("optional")
            opt.text = "true"
            elems.append(opt)
        target = Element("targetPackage")
        target.text = package
        elems.append(target)
        self.__add_config("artifactManagement", "rule", content=elems)

    def add_custom_option(self, optionstr, content):
        """
        Add custom configuration option

        optionstr -- XPath-like expression for specifying XMvn configuration
                     option location with '/' used as delimiter

                     example: buildSettings/compilerSource
        content -- XML content to be added to specified node. Can be just text, XML node or multiple nodes.

                   examples:
                   someText
                   <someNode>someText</someNode><someOtherNode/>
        """
        node_names = optionstr.split("/")
        confpath = self.__get_current_config()
        root = self.__init_xml()
        par = root
        for node in node_names:
            par = SubElement(par, node)

        try:
            #wrap content into something to allow text content
            inserted = "<root>{0}</root>".format(content);
            contentRoot = ET.fromstring(inserted);
            par.text = contentRoot.text
            for element in contentRoot:
                par.append(element)
        except lxml.etree.XMLSyntaxError:
            raise Exception("content is not valid content for XML node")
        self.__write_xml(confpath, root)



#!/usr/bin/python
# Copyright (c) 2013, Red Hat, Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Stanislav Ochotnicky <sochotnicky@redhat.com>
import gzip
import os.path

from lxml.etree import fromstring

from javapackages.artifact import Artifact

class DepmapLoadingException(Exception):
    pass

class DepmapInvalidException(Exception):
    pass

class Depmap(object):
    """
    Class for working with depmap files (dependency maps). These are files used
    by XMvn to provide mapping between Maven artifacts and file on the
    filesystem.

    Example usage:
    >>> d = Depmap('maven-idea-plugin.xml')
    >>> print d.get_java_requires()
    1.5
    >>> print d.get_provided_artifacts()[0]
    org.apache.maven.plugins:maven-idea-plugin:None:None:2.2
    """

    def __init__(self, path):
        self.__path = path
        self.__load_depmap(path)
        if self.__doc is None:
            raise DepmapLoadingException("Failed to load fragment {path} You have a problem".format(path=path))
        if not self.get_provided_mappings():
            raise DepmapLoadingException("Depmap {path} does not contain any provided artifacts ".format(path=path))

    def __load_depmap(self, fragment_path):
        with open(fragment_path) as f:
            try:
                gzf = gzip.GzipFile(os.path.basename(fragment_path),
                                    'rb',
                                    fileobj=f)
                data = gzf.read()
            except IOError:
                # not a compressed fragment, just rewind and read the data
                f.seek(0)
                data = f.read()

            self.__doc = fromstring(data)


    def is_compat(self):
         """Return true if depmap is for compatibility package

         This means package should have versioned provides"""

         provided_maps = self.get_provided_mappings()
         for m, l in provided_maps:
             if l.version:
                 return True
         return self.__doc.find(".//skipProvides") is not None

    def get_provided_artifacts(self):
        """Returns list of Artifact provided by given depmap."""
        artifacts = []
        for dep in self.__doc.findall('.//dependency'):
            artifact = dep.findall('./maven')
            if len(artifact) != 1:
                raise DepmapInvalidException("Multiple maven nodes in dependency")
            artifact = Artifact.from_xml_element(artifact[0])
            if not artifact.version:
                raise DepmapInvalidException("Depmap {path} does not have version in maven provides".format(path=self.__path))
            artifacts.append(artifact)
        return artifacts

    def get_provided_mappings(self):
        """Return list of (Artifact, Artifact) tuples.

        First part of returned tuple is Maven artifact identification
        Second part of returned tuple is local artifact identification
        """
        mappings = []
        for dep in self.__doc.findall('.//dependency'):
            m_artifact = dep.findall('./maven')
            if len(m_artifact) != 1:
                raise DepmapInvalidException("Multiple maven nodes in dependency")
            m_artifact = Artifact.from_xml_element(m_artifact[0])
            if not m_artifact.version:
                raise DepmapInvalidException("Depmap {path} does not have version in maven provides".format(path=self.__path))
            l_artifact = dep.findall('./jpp')
            if len(l_artifact) != 1:
                raise DepmapInvalidException("Multiple jpp nodes in dependency")
            l_artifact = Artifact.from_xml_element(l_artifact[0])
            mappings.append((m_artifact, l_artifact))
        return mappings

    def get_required_artifacts(self):
        """Returns list of Artifact required by given depmap."""
        artifacts = []
        for dep in self.__doc.findall('.//autoRequires'):
            artifact = Artifact.from_xml_element(dep)
            artifacts.append(artifact)
        return artifacts

    def get_skipped_artifacts(self):
        """Returns list of Artifact that were build but not installed"""
        artifacts = []
        for dep in self.__doc.findall('.//skippedArtifact'):
            artifact = Artifact.from_xml_element(dep)
            artifacts.append(artifact)
        return artifacts

    def get_java_requires(self):
        """Returns JVM version required by depmap or None"""
        jreq = self.__doc.find('.//requiresJava')
        if jreq is not None:
            jreq = jreq.text
        return jreq

    def get_java_devel_requires(self):
        """Returns JVM development version required by depmap or None"""
        jreq = self.__doc.find('.//requiresJavaDevel')
        if jreq is not None:
            jreq = jreq.text
        return jreq



#!/usr/bin/python
# Copyright (c) 2013, Red Hat, Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Stanislav Ochotnicky <sochotnicky@redhat.com>

from lxml.etree import ElementTree, XMLParser

class PomLoadingException(Exception):
    pass

class POM(object):
    """
    Class for querying basic information from pom.xml files used by Apache Maven
    """
    def __init__(self, path):
        et = ElementTree ()
        parser = XMLParser(remove_comments=True,
                           strip_cdata=True)
        self.__doc = et.parse(path, parser=parser)

        if self.__doc is None:
            raise PomLoadingException("Failed to load pom.xml. You have a problem")


    def __find(self, xpath):
        ret = self.__doc.xpath(xpath, namespaces=dict(pom='http://maven.apache.org/POM/4.0.0'))
        # perhaps there is no namespace?
        if len(ret) == 0:
            ret = self.__doc.xpath(xpath.replace('pom:',''))
        if len(ret) > 0:
            ret = ret[0]
        else:
            ret = None

        return ret

    @property
    def groupId(self):
        """
        Effective groupId of the pom Artifact taking into account parent groupId
        """
        gId = self.__find('./pom:groupId')
        if gId is None:
            gId = self.__find('./pom:parent/pom:groupId')
        if gId is None:
            gId = self.__find('/ivy-module/info')
            if gId is not None:
                gId = gId.attrib["organisation"]
            if gId is not None:
                return gId
        if gId is None:
            raise PomLoadingException("Unable to determine groupId")
        if len(gId) != 0:
            raise PomLoadingException("Unexpected child nodes under groupId")
        return gId.text.strip()

    @property
    def artifactId(self):
        """
        Effective artifactId of the pom Artifact
        """
        aId = self.__find('./pom:artifactId')
        if aId is None:
            aId = self.__find('/ivy-module/info')
            if aId is not None:
                aId = aId.attrib["module"]
            if aId is not None:
                return aId
        if aId is None:
            raise PomLoadingException("Unable to determine artifactID")
        if len(aId) != 0:
            raise PomLoadingException("Unexpected child nodes under artifactId")
        return aId.text.strip()

    @property
    def version(self):
        """
        Effective version of the pom Artifact taking into account parent
        version
        """
        version = self.__find('./pom:version')
        if version is None:
            version = self.__find('./pom:parent/pom:version')
        if version is None:
            version = self.__find('/ivy-module/info')
            if version is not None:
                version = version.attrib["revision"]
            if version is not None:
                return version
        if version is None:
            raise PomLoadingException("Unable to determine artifact version")
        if len(version) != 0:
            raise PomLoadingException("Unexpected child nodes under version")
        return version.text.strip()

    @property
    def packaging(self):
        """
        Packaging type of artifact or None if unspecified
        """
        p = self.__find('./pom:packaging')
        if p is not None:
            if len(p) != 0:
                raise PomLoadingException("Unexpected child nodes under packaging")
            return p.text.strip()
        p = self.__find('/ivy-module/info')
        if p is not None:
            return 'ivy'
        return None



#!/usr/bin/python
# Copyright (c) 2013, Red Hat, Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Stanislav Ochotnicky <sochotnicky@redhat.com>

import re
import sys

from lxml.etree import Element, SubElement, tostring

class ArtifactException(Exception):
    pass

class ArtifactFormatException(ArtifactException):
    pass

class ArtifactValidationException(ArtifactException):
    pass

class Artifact(object):
    """
    Simplified representation of Maven artifact for purpose of packaging

    Package consists mostly of simple properties and string formatting and
    loading functions to prevent code duplication elsewhere
    """

    def __init__(self, groupId, artifactId, extension="",
                 classifier="", version="", namespace=""):
        self.groupId = groupId.strip()
        self.artifactId = artifactId.strip()
        self.extension = extension.strip()
        self.classifier = classifier.strip()
        self.version = version.strip()
        self.namespace = namespace.strip()

    def __unicode__(self):
        return u"{gid}:{aid}:{ext}:{cls}:{ver}".format(gid=self.groupId,
                                                       aid=self.artifactId,
                                                       ext=self.extension,
                                                       cls=self.classifier,
                                                       ver=self.version)

    def __str__(self):
        return unicode(self).encode(sys.getfilesystemencoding())

    def get_rpm_str(self, versioned=False):
        """Return representation of artifact as used in RPM dependencies

        versioned -- return artifact string including version

        Example outputs:
        mvn(commons-logging:commons-logging)
        mvn(commons-logging:commons-logging:1.2) # versioned
        mvn(commons-logging:commons-logging:war:)
        mvn(commons-logging:commons-logging:war:1.2) # versioned
        mvn(commons-logging:commons-logging:war:test-jar:)
        mvn(commons-logging:commons-logging:war:test-jar:1.3) # versioned
        maven31-mvn(commons-logging:commons-logging)
        """
        namespace = "mvn"
        if self.namespace:
            namespace = self.namespace + "-mvn"

        mvnstr = "{gid}:{aid}".format(gid=self.groupId,
                                      aid=self.artifactId)

        if self.extension:
            mvnstr = mvnstr + ":{ext}".format(ext=self.extension)

        if self.classifier:
            if not self.extension:
                mvnstr = mvnstr + ":"
            mvnstr = mvnstr + ":{clas}".format(clas=self.classifier)

        if versioned:
            if not (self.version):
                raise ArtifactFormatException(
                    "Cannot create versioned string from artifact without version: {art}".format(art=str(self)))
            mvnstr = mvnstr + ":{ver}".format(ver=self.version)
        elif self.classifier or self.extension:
            mvnstr = mvnstr + ":"

        return "{namespace}({mvnstr})".format(namespace=namespace,
                                              mvnstr=mvnstr)


    def get_xml_element(self, root="artifact"):
        """
        Return XML Element node representation of the Artifact
        """
        root = Element(root)

        for key in ("artifactId", "groupId", "extension", "version",
                    "classifier", "namespace"):
            if hasattr(self, key) and getattr(self,key):
                item = SubElement(root, key)
                item.text = getattr(self, key)
        return root

    def get_xml_str(self, root="artifact"):
        """
        Return XML formatted string representation of the Artifact
        """
        root = self.get_xml_element(root)
        return tostring(root, pretty_print=True)

    def validate(self, allow_empty=True, allow_wildcards=True, allow_backref=True):
        """
        Function to validate current state of artifact with regards to
        wildcards, empty parts and backreference usage
        """
        all_empty = True
        wildcard_used = False
        backref_used = False
        backref_re = re.compile('@\d+')
        for key in ("artifactId", "groupId", "extension", "version",
                    "classifier", "namespace"):
            val = getattr(self, key)
            if not val:
                continue
            if val:
               all_empty = False
            if val.find('*') != -1:
                wildcard_used = True
            if backref_re.match(val):
                backref_used = True

        if not allow_empty and all_empty:
            raise ArtifactValidationException("All parts of artifact are empty")
        if not allow_wildcards and wildcard_used:
            raise ArtifactValidationException("Wildcard used in artifact")
        if not allow_backref and backref_used:
            raise ArtifactValidationException("Backreference used in artifact")
        return True

    @classmethod
    def merge_artifacts(cls, dominant, recessive):
        """
        Merge two artifacts into one. Information missing in dominant artifact will
        be copied from recessive artifact. Returns new merged artifact
        """
        ret = cls(dominant.groupId, dominant.artifactId, dominant.extension,
                  dominant.classifier, dominant.version, dominant.namespace)
        for key in ("artifactId", "groupId", "extension", "version",
                    "classifier", "namespace"):
            if not getattr(ret, key):
                setattr(ret, key, getattr(recessive, key))
        return ret

    @classmethod
    def from_xml_element(cls, xmlnode, namespace=""):
        """
        Create Artifact from xml.etree.ElementTree.Element as contained
        within pom.xml or a dependency map.
        """
        parts = {'groupId':'',
                 'artifactId':'',
                 'extension':'',
                 'classifier':'',
                 'version':''}

        for key in parts:
            node = xmlnode.find("./" + key)
            if node is not None and node.text is not None:
                parts[key] = node.text.strip()

        if not parts['groupId'] or not parts['artifactId']:
            raise ArtifactFormatException(
                "Empty groupId or artifactId encountered. "
                "This is a bug, please report it!")


        if not namespace:
            namespace = xmlnode.find('./namespace')
            if namespace is not None:
                namespace = namespace.text.strip()
            else:
                namespace = ""

        return cls(parts['groupId'], parts['artifactId'], parts['extension'],
                   parts['classifier'], parts['version'], namespace)

    @classmethod
    def from_mvn_str(cls, mvnstr, namespace=""):
        """
        Create Artifact from Maven-style definition

        The string should be in the format of:
           groupId:artifactId[:extension[:classifier]][:version]

        Where last part is always considered to be version unless empty
        """
        tup = mvnstr.split(":")

        # groupId and artifactId are always present
        if len(tup) < 2:
            raise ArtifactFormatException("Artifact string '{mvnstr}' does not "
                                          "contain ':' character. Can not parse".format(mvnstr=mvnstr))

        if len(tup) > 5:
            raise ArtifactFormatException("Artifact string '{mvnstr}' contains "
                                          "too many colons. Can not parse".format(mvnstr=mvnstr))

        groupId = tup[0]
        artifactId = tup[1]
        extension = tup[2] if len(tup) >= 4 else ""
        classifier = tup[3] if len(tup) >= 5 else ""
        version = tup[-1] if len(tup) >= 3 else ""

        return cls(groupId, artifactId, extension,
                   classifier, version, namespace)
